const assert = require('chai').assert;
const expect = require('chai').expect;
const Hand = require("pokersolver").Hand;

describe("PokerSolver hand calculator", function(){

  it("should return correct winning hand 'Ace high' for player_1", function(){
    table_cards = ["3s", "5d", "7h", "2h", "Td"];
    player_1 = ["As", "Qh"];
    player_2 = ["Ad", "Js"];
    hand_1 = Hand.solve(table_cards.concat(player_1));
    hand_2 = Hand.solve(table_cards.concat(player_2));
    winning_hands = Hand.winners([hand_1, hand_2]);
    expect(winning_hands.length).to.equal(1);
    expect(winning_hands[0]).to.equal(hand_1);
    expect(winning_hands.toString()).to.equal("As, Qh, 10d, 7h, 5d");
    expect(hand_1.name).to.equal("High Card");
    expect(hand_1.descr).to.equal("A High");
  });

  it("should return correct winning hand 'Pair' for player_2", function(){
    table_cards = ["3s", "5d", "7h", "2h", "Td"];
    player_1 = ["As", "Qh"];
    player_2 = ["3d", "Js"];
    hand_1 = Hand.solve(table_cards.concat(player_1));
    hand_2 = Hand.solve(table_cards.concat(player_2));
    winning_hands = Hand.winners([hand_1, hand_2]);
    expect(winning_hands.length).to.equal(1);
    expect(winning_hands[0]).to.equal(hand_2);
    expect(winning_hands.toString()).to.equal("3s, 3d, Js, 10d, 7h");
    expect(hand_2.name).to.equal("Pair");
    expect(hand_2.descr).to.equal("Pair, 3's");
  });

  it("should return correct winning hand 'Two Pair' for player_1", function(){
    table_cards = ["3s", "5d", "7h", "2h", "Td"];
    player_1 = ["5s", "2c"];
    player_2 = ["3d", "Js"];
    hand_1 = Hand.solve(table_cards.concat(player_1));
    hand_2 = Hand.solve(table_cards.concat(player_2));
    winning_hands = Hand.winners([hand_1, hand_2]);
    expect(winning_hands.length).to.equal(1);
    expect(winning_hands[0]).to.equal(hand_1);
    expect(winning_hands.toString()).to.equal("5d, 5s, 2h, 2c, 10d");
    expect(hand_1.name).to.equal("Two Pair");
    expect(hand_1.descr).to.equal("Two Pair, 5's & 2's");
  });

  it("should return correct winning hand 'Three of a kind' for player_2", function(){
    table_cards = ["3s", "3c", "7h", "2h", "Td"];
    player_1 = ["7d", "2c"];
    player_2 = ["3d", "Js"];
    hand_1 = Hand.solve(table_cards.concat(player_1));
    hand_2 = Hand.solve(table_cards.concat(player_2));
    winning_hands = Hand.winners([hand_1, hand_2]);
    expect(winning_hands.length).to.equal(1);
    expect(winning_hands[0]).to.equal(hand_2);
    expect(winning_hands.toString()).to.equal("3s, 3c, 3d, Js, 10d");
    expect(hand_2.name).to.equal("Three of a Kind");
    expect(hand_2.descr).to.equal("Three of a Kind, 3's");
  });

  it("should return correct winning hand 'Straight' for player_1", function(){
    table_cards = ["3s", "4c", "7h", "2h", "Td"];
    player_1 = ["5d", "6s"];
    player_2 = ["7d", "2c"];
    hand_1 = Hand.solve(table_cards.concat(player_1));
    hand_2 = Hand.solve(table_cards.concat(player_2));
    winning_hands = Hand.winners([hand_1, hand_2]);
    expect(winning_hands.length).to.equal(1);
    expect(winning_hands[0]).to.equal(hand_1);
    expect(winning_hands.toString()).to.equal("7h, 6s, 5d, 4c, 3s");
    expect(hand_1.name).to.equal("Straight");
    expect(hand_1.descr).to.equal("Straight, 7 High");
  });

  it("should return correct winning hand 'Flush' for player_2", function(){
    table_cards = ["3s", "4c", "7h", "2h", "Th"];
    player_1 = ["2d", "2c"];
    player_2 = ["5h", "Ah"];
    hand_1 = Hand.solve(table_cards.concat(player_1));
    hand_2 = Hand.solve(table_cards.concat(player_2));
    winning_hands = Hand.winners([hand_1, hand_2]);
    expect(winning_hands.length).to.equal(1);
    expect(winning_hands[0]).to.equal(hand_2);
    expect(winning_hands.toString()).to.equal("Ah, 10h, 7h, 5h, 2h");
    expect(hand_2.name).to.equal("Flush");
    expect(hand_2.descr).to.equal("Flush, Ah High");
  });

  it("should return correct winning hand 'Full House' for player_1", function(){
    table_cards = ["3s", "3c", "7h", "2h", "Th"];
    player_1 = ["3d", "7c"];
    player_2 = ["5h", "Ah"];
    hand_1 = Hand.solve(table_cards.concat(player_1));
    hand_2 = Hand.solve(table_cards.concat(player_2));
    winning_hands = Hand.winners([hand_1, hand_2]);
    expect(winning_hands.length).to.equal(1);
    expect(winning_hands[0]).to.equal(hand_1);
    expect(winning_hands.toString()).to.equal("3s, 3c, 3d, 7h, 7c");
    expect(hand_1.name).to.equal("Full House");
    expect(hand_1.descr).to.equal("Full House, 3's over 7's");
  });

  it("should return correct winning hand 'Four of a Kind' for player_2", function(){
    table_cards = ["3s", "3c", "7h", "2h", "Th"];
    player_1 = ["2d", "7c"];
    player_2 = ["3h", "3d"];
    hand_1 = Hand.solve(table_cards.concat(player_1));
    hand_2 = Hand.solve(table_cards.concat(player_2));
    winning_hands = Hand.winners([hand_1, hand_2]);
    expect(winning_hands.length).to.equal(1);
    expect(winning_hands[0]).to.equal(hand_2);
    expect(winning_hands.toString()).to.equal("3s, 3c, 3h, 3d, 10h");
    expect(hand_2.name).to.equal("Four of a Kind");
    expect(hand_2.descr).to.equal("Four of a Kind, 3's");
  });

  it("should return correct winning hand 'Straight Flush' for player_1", function(){
    table_cards = ["7h", "8h", "9h", "2d", "Td"];
    player_1 = ["6h", "Th"];
    player_2 = ["3h", "3d"];
    hand_1 = Hand.solve(table_cards.concat(player_1));
    hand_2 = Hand.solve(table_cards.concat(player_2));
    winning_hands = Hand.winners([hand_1, hand_2]);
    expect(winning_hands.length).to.equal(1);
    expect(winning_hands[0]).to.equal(hand_1);
    //
    // NOTE: this is incorrect, it should only return the first 5 cards, on a straight
    // flush it is returning all 7 cards
    // also Th is returning 10h, for consistancy it should return Th
    // These issues are fixed in GamePlay when returning winning cards
    // Also we return an array of winning cards not a string
    //
    expect(winning_hands.toString()).to.equal("10h, 9h, 8h, 7h, 6h, 10d, 2d");
    expect(hand_1.name).to.equal("Straight Flush");
    expect(hand_1.descr).to.equal("Straight Flush, 10h High");
  });

  it("should return correct winning hand 'Royal Flush' for player_2", function(){
    table_cards = ["7h", "8h", "Ad", "Jd", "Td"];
    player_1 = ["6h", "Th"];
    player_2 = ["Kd", "Qd"];
    hand_1 = Hand.solve(table_cards.concat(player_1));
    hand_2 = Hand.solve(table_cards.concat(player_2));
    winning_hands = Hand.winners([hand_1, hand_2]);
    expect(winning_hands.length).to.equal(1);
    expect(winning_hands[0]).to.equal(hand_2);
    // Again, incorrectly returning 7 cards
    expect(winning_hands.toString()).to.equal("Ad, Kd, Qd, Jd, 10d, 8h, 7h");
    // Name is returning incorrectly as Straight Flush, however description is returning Royal Flush
    expect(hand_2.name).to.equal("Straight Flush");
    expect(hand_2.descr).to.equal("Royal Flush");
  });

});
